### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### These are the tests for the crop growth models.
###

import Unitful

@testset for paramfile in [
    "test_parameters_almass.toml",
    "test_parameters_simplecrop.toml",
    "test_parameters_aquacrop.toml",
]
    parampath = joinpath(pkgdir(Persefone), "test", paramfile)
    @testset "Model initialisation" begin
        model = initialise(parampath)
        @test model isa AgricultureModel
    end
    @testset "Time step" begin
        model = initialise(parampath)
        stepsimulation!(model)
        @test model isa AgricultureModel
        # TODO: test stepagent!(), sow!(), harvest!()
    end
end

@testset "Crop model ALMaSS" begin
    ct = Ps.ALMaSS.CropType("olive tree", "no_crop_group",
                            missing, missing, missing, missing, missing, missing, missing)
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    mature = false
    events = Ps.Management[]
    fp = FarmPlot(id, pixels, farmer,
                  Ps.ALMaSS.CropState(ct, Ps.ALMaSS.janfirst, 0.0, 0.0m, 0.0, 0.0, mature, events))
    @test fp isa FarmPlot
    @test fp isa FarmPlot{Ps.ALMaSS.CropState}
    @test croptype(fp) isa Ps.ALMaSS.CropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test cropyield(fp) isa Float64
end

@testset "Crop model SimpleCrop" begin
    ct = Ps.SimpleCrop.CropType("olive tree", "no_crop_group", AnnualDate(4, 1), AnnualDate(5, 1))
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    fp = FarmPlot(id, pixels, farmer, Ps.SimpleCrop.CropState(ct, 0.0cm))
    @test fp isa FarmPlot
    @test fp isa FarmPlot{Ps.SimpleCrop.CropState}
    @test croptype(fp) isa Ps.SimpleCrop.CropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test cropyield(fp) isa Float64
end

@testset "Crop model AquaCrop" begin
    ct = Ps.AquaCropType("olive tree", "no_crop_group", AnnualDate(4, 1), AnnualDate(5, 1))
    id = 0
    pixels = [(0, 0)]
    farmer = 0
    fp = FarmPlot(id, pixels, farmer, Ps.AquaCropState(ct))
    @test fp isa FarmPlot
    @test fp isa FarmPlot{Ps.AquaCropState}
    @test croptype(fp) isa Ps.AquaCropType
    @test cropname(fp) isa String
    @test cropheight(fp) isa Length{Float64}
    @test cropcover(fp) isa Float64
    @test Unitful.dimension(cropyield(fp)) == Unitful.𝐌 * Unitful.𝐋^-2
end
